---
title: 'CLODE: A Python/C++ package for large-scale ODE simulation'
tags:
  - Python
  - C++
  - OpenCL
  - ODE
  - GPU
authors:
  - name: Patrick Fletcher
    affiliation: 1
  - name: Wolf Byttner
    affiliation: 2
affiliations:
  - name: National Institute of Health, USA
    index: 1
  - name: University of Exeter, UK
    index: 2
date: 2023-03-14
bibliography: paper.bib
---

# Summary

CLODE is a Python/C++ package for large-scale grid-search and
analysis of large Ordinary Differential Equation (ODE) systems.
It uses OpenCL to parallelize ODE integration on a GPU, letting
users run over 100,000 simulations per second. The package is
designed to work with XPPAUT files, which is commonly used in
computational neuroscience to define ODE systems, but can also
be used as a standalone solver with python- or OpenCL-defined
right-hand side equations.

CLODE was developed to study populations of ODEs in [@fletcher:2016]
and has been used in [@fletcher:2017]. It is currently being used
for several projects in computational neuroscience and endocrinology.
The Python interface was developed using [@pybind11].

# Statement of need

CLODE specialises on the efficient generation of large-scale databases
of ODE simulations, generated by parameter grid-search. Because of
the immense memory requirements of such simulations, CLODE is designed
to capture relevant features of the ODEs on the fly, without storing
the full trajectory data. This reduces memory requirements by over 99 %,
and reduces the time required to run simulations by several orders of
magnitude, making previously infeasible simulations possible on a
typical desktop computer.

A large number of research questions concern ODEs, and scientists often
want to understand the aggregate influence of their parameters.
For meaningful statistical analysis, researchers need to run
millions to billions of simulations, requiring special hardware
and computational expertise. CLODE is designed to make this process
accessible to researchers without a background in high-performance
computing and to fit within the time  constraints of a typical
research project.

## Alternatives considered

There are several other ODE solvers available in Python, but none
of them are designed for on-the-fly feature extraction. These include:

- SciPy's `odeint` and `solve_ivp` functions, which are high-level
    python wrappers for the Fortran-based ODEPACK library [@2020SciPy-NMeth].
- PyDSTool, which is designed for symbolic manipulation of ODEs.
- PyDDE, which is designed for delay differential equations.
- torchdiffeq, which is designed for deep learning [@chen2018neuralode].
- diffeqpy, which is a Python wrapper for the Julia package
    DifferentialEquations.jl [@DifferentialEquations.jl-2017].
- Packages like CudaPyInt and torchode, which are designed for
    GPU acceleration of smaller systems [@lienen2022torchode].

Additionally, faster solvers could be implemented in several helper languages:

- numba, which is a just-in-time compiler for Python.
    The speedup of numba is however insufficient.
- TensorFlow, and Torch, which are deep learning libraries
    that can be used to solve ODEs. But neither have the low-level
    memory control needed for on-the-fly feature extraction.

Traditional ODE solvers in Python and Matlab, such as SciPy's
`odeint` and Matlab's `ode45`, are not designed for large-scale
simulations and take tens to hundreds of milliseconds to integrate
typical ODEs with up to a dozen state variables. CLODE is designed
to run on a GPU, and can solve ODEs in parallel, with each ODE
taking microseconds to integrate.

CLODE can also capture ODE features on the fly, without requiring
explicit trajectories, thus reducing memory requirements by over 99 %
for large-scale simulations with millions of ODEs. This lets researchers
run experiments on typical desktop hardware, without requiring dedicated
HPC clusters.

# Current applications and future work

CLODE is currently being used to study the dynamics of cells in the
pituitary gland, a complex endocrine organ that controls hormone
secretion across many organs in the body. CLODE is used to simulate
the electrical activity of pituitary cells, to better understand
how different cell membrane ion channels contribute to the
cell's internal calcium dynamics.

The code base is currently being extended to support
XPPAUT files in Python. XPPAUT is a dynamical systems modeling
program that is widely used in ODE research.

# Acknowledgements

We would like to thank Joel Tabak and Richard Bertram for extensively testing
CLODE. We would also like to thank the reviewers for their helpful comments.

# References